@@ -1,10 +1,13 @@
+//! Implements AMQP1.0 Connection
+
 use std::{cmp::min, collections::BTreeMap, convert::TryInto, io};
 
 use async_trait::async_trait;
 
 use fe2o3_amqp_types::{
     definitions::{self, AmqpError},
-    performatives::{Begin, ChannelMax, Close, End, MaxFrameSize, Open},
+    performatives::{Begin, Close, End, Open},
+    states::ConnectionState,
 };
 use futures_util::{Sink, SinkExt};
 use slab::Slab;
@@ -19,69 +22,100 @@ use crate::{
     control::ConnectionControl,
     endpoint,
     frames::amqp::{Frame, FrameBody},
-    session::SessionFrame,
-    session::{Session, SessionFrameBody, SessionIncomingItem},
+    session::frame::{SessionFrame, SessionFrameBody, SessionIncomingItem},
+    session::Session,
 };
 
 use self::{builder::WithoutContainerId, engine::SessionId};
 
-pub mod builder;
-pub mod engine;
+mod builder;
+pub use builder::*;
+
+pub(crate) mod engine;
+
 mod error;
 pub mod heartbeat;
 pub use error::*;
 
-#[derive(Debug, Clone)]
-pub enum ConnectionState {
-    Start,
+/// Default max-frame-size
+pub const DEFAULT_MAX_FRAME_SIZE: u32 = 256 * 1024;
 
-    HeaderReceived,
-
-    HeaderSent,
-
-    HeaderExchange,
-
-    OpenPipe,
-
-    OpenClosePipe,
-
-    OpenReceived,
-
-    OpenSent,
-
-    ClosePipe,
-
-    Opened,
-
-    CloseReceived,
-
-    CloseSent,
-
-    Discarding,
-
-    End,
-}
+/// Default channel-max
+pub const DEFAULT_CHANNEL_MAX: u16 = 255;
 
+/// A handle to the [`Connection`] event loop.
+///
+/// Dropping the handle will also stop the [`Connection`] event loop
+#[derive(Debug)]
 pub struct ConnectionHandle {
     pub(crate) control: Sender<ConnectionControl>,
     handle: JoinHandle<Result<(), Error>>,
 
     // outgoing channel for session
     pub(crate) outgoing: Sender<SessionFrame>,
-    // session_control: Sender<SessionControl>,
+}
+
+impl Drop for ConnectionHandle {
+    fn drop(&mut self) {
+        let _ = self.control.try_send(ConnectionControl::Close(None));
+    }
 }
 
 impl ConnectionHandle {
+    /// Checks if the underlying event loop has stopped
+    pub fn is_closed(&self) -> bool {
+        self.control.is_closed()
+    }
+
+    /// Close the connection
+    ///
+    /// # Panics
+    ///
+    /// Panics if this is called after executing any of [`close`](#method.close),
+    /// [`close_with_error`](#method.close_with_error) or [`on_close`](#method.on_close).
+    /// This will cause the JoinHandle to be polled after completion, which causes a panic.
     pub async fn close(&mut self) -> Result<(), Error> {
         // If sending is unsuccessful, the `ConnectionEngine` event loop is
         // already dropped, this should be reflected by `JoinError` then.
         let _ = self.control.send(ConnectionControl::Close(None)).await;
+        self.on_close().await
+    }
+
+    /// Close the connection with an error
+    ///
+    /// # Panics
+    ///
+    /// Panics if this is called after executing any of [`close`](#method.close),
+    /// [`close_with_error`](#method.close_with_error) or [`on_close`](#method.on_close).
+    /// This will cause the JoinHandle to be polled after completion, which causes a panic.
+    pub async fn close_with_error(
+        &mut self,
+        error: impl Into<definitions::Error>,
+    ) -> Result<(), Error> {
+        // If sending is unsuccessful, the `ConnectionEngine` event loop is
+        // already dropped, this should be reflected by `JoinError` then.
+        let _ = self
+            .control
+            .send(ConnectionControl::Close(Some(error.into())))
+            .await;
+        self.on_close().await
+    }
+
+    /// Returns when the underlying event loop has stopped
+    ///
+    /// # Panics
+    ///
+    /// Panics if this is called after executing any of [`close`](#method.close),
+    /// [`close_with_error`](#method.close_with_error) or [`on_close`](#method.on_close).
+    /// This will cause the JoinHandle to be polled after completion, which causes a panic.
+    pub async fn on_close(&mut self) -> Result<(), Error> {
         match (&mut self.handle).await {
             Ok(res) => res,
             Err(e) => Err(Error::JoinError(e)),
         }
     }
 
+    /// Allocte (channel, session_id) for a new session
     pub(crate) async fn allocate_session(
         &mut self,
         tx: Sender<SessionIncomingItem>,
@@ -101,13 +135,117 @@ impl ConnectionHandle {
         })?;
         result
     }
-
-    // pub(crate) async fn drop_session(&mut self, session_id: SessionId) -> Result<(), Error> {
-    //     self.control.send(ConnectionControl::DropSession(session_id)).await?;
-    //     Ok(())
-    // }
 }
 
+/// An AMQP 1.0 Connection.
+///
+/// # Open a new [`Connection`] with default configuration
+///
+/// Below is an example with a local broker (
+/// [`TestAmqpBroker`](https://github.com/Azure/amqpnetlite/releases/download/test_broker.1609/TestAmqpBroker.zip))
+/// listening on the localhost. The broker is executed with the following command
+///
+/// ```powershell
+/// ./TestAmqpBroker.exe amqp://localhost:5672 /creds:guest:guest /queues:q1
+/// ```
+///
+/// ```rust, ignore
+/// let connection = Connection::open(
+///     "connection-1", // container id
+///     "amqp://guest:guest@localhost:5672" // url with username and password
+/// ).await.unwrap();
+/// ```
+///
+/// ## Default configuration
+///
+/// | Field | Default Value |
+/// |-------|---------------|
+/// |`max_frame_size`| [`DEFAULT_MAX_FRAME_SIZE`] |
+/// |`channel_max`| [`DEFAULT_CHANNEL_MAX`] |
+/// |`idle_time_out`| `None` |
+/// |`outgoing_locales`| `None` |
+/// |`incoming_locales`| `None` |
+/// |`offered_capabilities`| `None` |
+/// |`desired_capabilities`| `None` |
+/// |`Properties`| `None` |
+/// 
+/// ## Customize configuration with [`Builder`]
+///
+/// The example above creates a connection with the default configuration. If the user needs to customize the
+/// configuration, the connection [`Builder`] should be used.
+///
+/// ```rust, ignore
+/// let connection = Connection::builder()
+///     .container_id("connection-1")
+///     .max_frame_size(4096)
+///     .channel_max(64)
+///     .idle_time_out(50_000 as u32)
+///     .open("amqp://guest:guest@localhost:5672")
+///     .await.unwrap();
+/// ```
+///
+/// ## TLS
+///
+/// TLS is supported with `rustls` by setting the url scheme to `"amqps"`. 
+/// The user must use the builder to supply a custom `ClientConfig` for the TLS connection. 
+/// If there is no custom `ClientConfig`, the TLS handshake will use the the following
+/// config.
+/// 
+/// ```rust, ignore
+/// ClientConfig::builder()
+///     .with_safe_defaults()
+///     .with_root_certificates(RootCertStore::empty())
+///     .with_no_client_auth()
+/// ```
+/// 
+/// The code example below shows TLS connection without supplying a custom `ClientConfig`
+/// 
+/// ```rust, ignore
+/// let connection = Connection::open("connection-1", "amqps://localhost:5672").await.unwrap();
+/// ```
+/// 
+/// The code example below shows how a user can supply a custom `ClientConfig` using builder.
+///
+/// ```rust, ignore
+/// // Set custom `ClientConfig`
+/// let connection = Connection::builder()
+///     .container_id("connection-1")
+///     .client_config(config)
+///     .open("amqps://localhost:5672")
+///     .await.unwrap();
+/// ```
+///
+/// ## SASL
+///
+/// If `username` and `password` are supplied with the url, the connection negotiation will start with
+/// SASL PLAIN negotiation. Other than filling `username` and `password` in the url, one could also
+/// supply the information with `sasl_profile` field of the [`Builder`]. Please note that the SASL profile
+/// found in the url will override whatever `SaslProfile` supplied to the [`Builder`].
+///
+/// The examples below shows two ways of starting the connection with SASL negotiation.
+/// 
+/// 1. Start SASL negotiation with SASL PLAIN profile extracted from the url
+///
+///     ```rust,ignore
+///     let connection = Connection::open("connection-1", "amqp://guest:guest@localhost:5672").await.unwrap();
+///     ```
+/// 
+/// 2. Start SASL negotiation with the builder. Please note that tf the url contains `username` and `password`, 
+/// the profile supplied to the builder will be overriden.
+/// 
+///     ```
+///     // This is equivalent to the line above
+///     let profile = SaslProfile::Plain {
+///         username: "guest".to_string(),
+///         password: "guest".to_string()
+///     };
+///     let connection = Connection::builder()
+///         .container_id("connection-1")
+///         .sasl_profile(profile)
+///         .open("amqp://localhost:5672")
+///         .await.unwrap();
+///     ```
+///
 #[derive(Debug)]
 pub struct Connection {
     control: Sender<ConnectionControl>,
@@ -128,20 +266,63 @@ pub struct Connection {
 
 /* ------------------------------- Public API ------------------------------- */
 impl Connection {
+    /// Creates a Builder for [`Connection`]
     pub fn builder<'a>() -> builder::Builder<'a, WithoutContainerId> {
         builder::Builder::new()
     }
 
+    /// Negotiate and open a [`Connection`] with the default configuration
+    ///
+    /// # Default configuration
+    ///
+    /// | Field | Default Value |
+    /// |-------|---------------|
+    /// |`max_frame_size`| [`DEFAULT_MAX_FRAME_SIZE`] |
+    /// |`channel_max`| [`DEFAULT_CHANNEL_MAX`] |
+    /// |`idle_time_out`| `None` |
+    /// |`outgoing_locales`| `None` |
+    /// |`incoming_locales`| `None` |
+    /// |`offered_capabilities`| `None` |
+    /// |`desired_capabilities`| `None` |
+    /// |`Properties`| `None` |
+    ///
+    /// The negotiation depends on the url supplied.
+    ///
+    /// # Raw AMQP
+    ///
+    /// ```rust, ignore
+    /// let connection = Connection::open("connection-1", "amqp://localhost:5672").await.unwrap();
+    /// ```
+    /// 
+    /// # TLS
+    /// 
+    /// This will start TLS negotiation with the following `ClientConfig`
+    /// 
+    /// ```rust, ignore
+    /// ClientConfig::builder()
+    ///     .with_safe_defaults()
+    ///     .with_root_certificates(RootCertStore::empty())
+    ///     .with_no_client_auth()
+    /// ```
+    /// 
+    /// ```rust, ignore
+    /// let connection = Connection::open("connection-1", "amqps://localhost:5672").await.unwrap();
+    /// ```
+    ///
+    /// # SASL
+    ///
+    /// Start SASL negotiation with SASL PLAIN profile extracted from the url
+    /// 
+    /// ```rust, ignore
+    /// let connection = Connection::open("connection-1", "amqp://guest:guest@localhost:5672").await.unwrap();
+    /// ```
+    ///
     pub async fn open(
-        container_id: String, // TODO: default container id? random uuid-ish
-        max_frame_size: impl Into<MaxFrameSize>, // TODO: make this use default?
-        channel_max: impl Into<ChannelMax>, // make this use default?
+        container_id: impl Into<String>, // TODO: default container id? random uuid-ish
         url: impl TryInto<Url, Error = url::ParseError>,
-    ) -> Result<ConnectionHandle, Error> {
+    ) -> Result<ConnectionHandle, OpenError> {
         Connection::builder()
             .container_id(container_id)
-            .max_frame_size(max_frame_size)
-            .channel_max(channel_max)
             .open(url)
             .await
     }
@@ -231,7 +412,7 @@ impl endpoint::Connection for Connection {
             ConnectionState::HeaderExchange => self.local_state = ConnectionState::OpenReceived,
             ConnectionState::OpenSent => self.local_state = ConnectionState::Opened,
             ConnectionState::ClosePipe => self.local_state = ConnectionState::CloseSent,
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)),
         }
 
         // set channel_max to mutually acceptable
@@ -248,7 +429,7 @@ impl endpoint::Connection for Connection {
         match &self.local_state {
             ConnectionState::Opened => {}
             // TODO: what about pipelined
-            _ => return Err(AmqpError::IllegalState.into()), // TODO: what to do?
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)), // TODO: what to do?
         }
 
         match begin.remote_channel {
@@ -256,10 +437,10 @@ impl endpoint::Connection for Connection {
                 let session_id = self
                     .session_by_outgoing_channel
                     .get(&outgoing_channel)
-                    .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?;
+                    .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?; // Close with error NotFound
 
                 if self.session_by_incoming_channel.contains_key(&channel) {
-                    return Err(AmqpError::NotAllowed.into()); // TODO: this is probably not how not allowed should be used?
+                    return Err(Error::amqp_error(AmqpError::NotAllowed, None)); // TODO: this is probably not how not allowed should be used?
                 }
                 self.session_by_incoming_channel
                     .insert(channel, *session_id);
@@ -268,7 +449,7 @@ impl endpoint::Connection for Connection {
                 let tx = self
                     .local_sessions
                     .get_mut(*session_id)
-                    .ok_or_else(|| AmqpError::NotFound)?;
+                    .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?;
                 let sframe = SessionFrame::new(channel, SessionFrameBody::Begin(begin));
                 tx.send(sframe).await?;
             }
@@ -276,10 +457,11 @@ impl endpoint::Connection for Connection {
                 // If a session is locally initiated, the remote-channel MUST NOT be set. When an endpoint responds
                 // to a remotely initiated session, the remote-channel MUST be set to the channel on which the
                 // remote session sent the begin.
-                return Err(Error::AmqpError {
-                    condition: AmqpError::NotAllowed,
-                    description: Some("remote-channel is not set".to_string()),
-                });
+                // TODO: allow remotely initiated session
+                return Err(Error::amqp_error(
+                    AmqpError::NotImplemented,
+                    Some("Remotely initiazted session is not supported yet".to_string()),
+                )); // Close with error NotImplemented
             }
         }
 
@@ -292,7 +474,7 @@ impl endpoint::Connection for Connection {
         trace!(channel, frame = ?end);
         match &self.local_state {
             ConnectionState::Opened => {}
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)),
         }
 
         // Forward to session
@@ -301,10 +483,10 @@ impl endpoint::Connection for Connection {
         let session_id = self
             .session_by_incoming_channel
             .remove(&channel)
-            .ok_or_else(|| AmqpError::NotFound)?;
+            .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?;
         self.local_sessions
             .get_mut(session_id)
-            .ok_or_else(|| AmqpError::NotFound)?
+            .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?
             .send(sframe)
             .await?;
 
@@ -313,11 +495,7 @@ impl endpoint::Connection for Connection {
 
     /// Reacting to remote Close frame
     #[instrument(name = "RECV", skip_all)]
-    async fn on_incoming_close(
-        &mut self,
-        channel: u16,
-        close: Close,
-    ) -> Result<Option<definitions::Error>, Self::Error> {
+    async fn on_incoming_close(&mut self, channel: u16, close: Close) -> Result<(), Self::Error> {
         trace!(channel, frame=?close);
 
         match &self.local_state {
@@ -326,10 +504,13 @@ impl endpoint::Connection for Connection {
                 self.control.send(ConnectionControl::Close(None)).await?;
             }
             ConnectionState::CloseSent => self.local_state = ConnectionState::End,
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)),
         };
 
-        Ok(close.error)
+        match close.error {
+            Some(error) => Err(Error::Remote(error)),
+            None => Ok(()),
+        }
     }
 
     #[instrument(name = "SEND", skip_all)]
@@ -348,7 +529,7 @@ impl endpoint::Connection for Connection {
             ConnectionState::HeaderExchange => self.local_state = ConnectionState::OpenSent,
             ConnectionState::OpenReceived => self.local_state = ConnectionState::Opened,
             ConnectionState::HeaderSent => self.local_state = ConnectionState::OpenPipe,
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)),
         }
 
         Ok(())
@@ -369,7 +550,7 @@ impl endpoint::Connection for Connection {
     fn on_outgoing_end(&mut self, channel: u16, end: End) -> Result<Frame, Self::Error> {
         self.session_by_outgoing_channel
             .remove(&channel)
-            .ok_or_else(|| AmqpError::NotFound)?;
+            .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?;
         let frame = Frame::new(channel, FrameBody::End(end));
         Ok(frame)
     }
@@ -394,7 +575,7 @@ impl endpoint::Connection for Connection {
             ConnectionState::CloseReceived => self.local_state = ConnectionState::End,
             ConnectionState::OpenSent => self.local_state = ConnectionState::ClosePipe,
             ConnectionState::OpenPipe => self.local_state = ConnectionState::OpenClosePipe,
-            _ => return Err(AmqpError::IllegalState.into()),
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)),
         }
         Ok(())
     }
