@@ -1,3 +1,5 @@
+//! Common utilities
+
 use futures_util::Future;
 use std::ops::Deref;
 use std::{pin::Pin, task::Poll, time::Duration};
@@ -10,13 +12,13 @@ mod producer;
 pub use consumer::*;
 pub use producer::*;
 
-pub enum Running {
+pub(crate) enum Running {
     Continue,
     Stop,
 }
 
 #[derive(Debug)]
-pub struct IdleTimeout {
+pub(crate) struct IdleTimeout {
     delay: Pin<Box<Sleep>>,
     duration: Duration,
 }
@@ -27,10 +29,6 @@ impl IdleTimeout {
         Self { delay, duration }
     }
 
-    pub fn duration(&self) -> Duration {
-        self.duration
-    }
-
     pub fn reset(&mut self) {
         let now = Instant::now();
         let next = now + self.duration;
@@ -49,7 +47,8 @@ impl Future for IdleTimeout {
 
 /// An custom type to make a field immutable to
 /// prevent accidental mutations
-pub struct Constant<T> {
+#[derive(Debug)]
+pub(crate) struct Constant<T> {
     value: T,
 }
 
@@ -71,7 +70,6 @@ impl<T> Deref for Constant<T> {
     }
 }
 
-/// Shared type state for builder
+/// Shared type state for builders
+#[derive(Debug)]
 pub struct Uninitialized {}
-
-// pub struct Initialized {}
